package master;

import main.Song_J2ME;
import main.Unit_J2ME;
import db.DB_J2ME;
import filter.QueryFilter_J2ME;
import gui.GUI_J2ME;
import storage.Storage_J2ME;
import main.CMD_Enum;
import translator.CmdTranslator_J2ME;

/**
 * 
 * @author pete
 */
public abstract class Master_J2ME implements Unit_J2ME {
	public static final String VOLUME = " volume: ";
	/** link to database */
	DB_J2ME db_lnk;
	/** link to stoarge */
	Storage_J2ME storage_lnk;
	/** link to gui */
	GUI_J2ME gui_lnk;
	/** link to command translator */
	CmdTranslator_J2ME cmd_translator_lnk;
	private QueryFilter_J2ME qf;

	public static final String FILE = " file: ";
	public static final String TIME = " Time: ";
	public static final String ID = " Id: ";
	public static final String TITLE = " Title: ";
	public static final String ARTIST = " Artist: ";
	public static final String ALBUM = " Album: ";
	public static final String YEAR = " Date: ";
	public static final String POS = " Pos: ";

	/** Construct new Master */
	public Master_J2ME() {
		db_lnk = new DB_J2ME(this);
		storage_lnk = new Storage_J2ME(this);
		cmd_translator_lnk = new CmdTranslator_J2ME(this);
		gui_lnk = new GUI_J2ME(this);
		qf = cmd_translator_lnk.getQueryFilter();
	}

	/**
	 * return current mpd volume
	 * 
	 * @return volume, int, 0-100
	 * 
	 * */
	public int getVolume() {
		String[] out = sendCommand(CMD_Enum.CMD_STATUS, VOLUME, "");
		if (out == null) {
			return -1;
		} else return Integer.parseInt(out[0]);
	}

	public String[] getArtists() {
		System.out.println("---get:artits---");
		return sendCommand(CMD_Enum.CMD_LIST, ARTIST, CmdTranslator_J2ME.ARTIST);
	}

	public String[] getAlbums(String artist) {
		System.out.println("---get:albums---");
		return sendCommand(CMD_Enum.CMD_LIST, ALBUM, CmdTranslator_J2ME.ALBUM + " " + "\"" + artist + "\"");
	}

	/**
	 * 
	 * initiate Master
	 * 
	 * */

	public void init() {
		db_lnk.init();
		storage_lnk.init();
		cmd_translator_lnk.init();
		if (qf.isReady)
			updateDb();
		gui_lnk.init();
	}

	/**
	 * 
	 * initiate load process
	 * 
	 * */
	public void initiateLoad() {
		db_lnk.initiateLoad();
		cmd_translator_lnk.initiateLoad();
		qf.initiateLoad();
		gui_lnk.initiateLoad();
	}

	/**
	 * 
	 * initiate save process
	 * 
	 * */

	public void initiateSave() {
		db_lnk.initiateSave();
		cmd_translator_lnk.initiateSave();
		qf.initiateSave();
		gui_lnk.initiateSave();
	}

	/**
	 * 
	 * its killing me
	 * 
	 * */
	public void die() {
		gui_lnk.die();
		db_lnk.die();
		cmd_translator_lnk.die();
	}

	/**
	 * 
	 * load [] by key
	 * 
	 * @param key
	 *            to saved file
	 * 
	 * @return saved array
	 * 
	 * */

	public String[] load(String key) {
		return MasterExtender_J2ME
				.ByteArrayToStringArray(storage_lnk.load(key));
	}
	public Song_J2ME getSong (int pos) {
		return db_lnk.songs[pos];
	}

	public String[] getUrls() {
		return qf.urls;
	}

	public void setUrl(String url) {
		for (int i = 0; i < qf.urls.length-1; i++)
			qf.urls[i] = qf.urls[i+1];
		qf.urls[qf.urls.length-1] = url;
		qf.cUrl = qf.urls.length-1;//всегда!!
		qf.setConnection();
		updateDb();
	}

	/**
	 * 
	 * send command to player
	 * 
	 * @param cmd
	 *            command id
	 * @param key
	 *            crop key
	 * @param value
	 *            command parametr
	 * 
	 * @return command return, {OK} if suckness;
	 * 
	 * */

	private String[] sendCommand(int cmd, String key, String value) {
		System.out.println("send cmd:" + cmd + "," + key + "," + value + ";");
		if (qf.isReady) {
			if (cmd == CMD_Enum.CMD_GET_PLAYLIST) {
				String[] all = cmd_translator_lnk.send(cmd, key, value);
				db_lnk.songs = parseStringsToSongs(all);
				db_lnk.length = all.length / QueryFilter_J2ME.TAG_SIZE;
				String[] out = { "OK" };
				return out;
			} else {
				return cmd_translator_lnk.send(cmd, key, value);
			}
		} return null;
	}

	public Song_J2ME[] getSong(String type, String what) {
		if (type == "" && what == "") {
			return getAll();
		} else if (what.indexOf('*') != -1)
			return parseStringsToSongs(sendCommand(CMD_Enum.CMD_FIND, "", type
					+ " "));
		else
			return parseStringsToSongs(sendCommand(CMD_Enum.CMD_FIND, "", type
					+ " \"" + what + "\""));

	}

	public void addByUrl(String url) {
		sendCommand(CMD_Enum.CMD_ADD, "", "\"" + url + "\"");
	}

	/**
	 * 
	 * return length of current playlist
	 * 
	 * @return length
	 * 
	 * */

	public int getPlaylistLength() {
		return db_lnk.length;
	}

	private void addAll() {
		sendCommand(CMD_Enum.CMD_ADD, "", "/");
	}

	private Song_J2ME[] getAll() {
		return parseStringsToSongs(sendCommand(CMD_Enum.CMD_LIST_ALL_INFO, "",
				"/"));
	}

	/**
	 * @return part of playlist
	 * */
	public Song_J2ME[] getPlaylist(int offset, int length) {
		System.out.println("getPlaylist(" + offset + "," + length + ")");
		if (offset < 0 && offset >= db_lnk.length) {
			return null;
		}
		if (length == -1) {
			length = db_lnk.length - offset;
		} else {
			length = (offset + db_lnk.length) % db_lnk.length;
		}
		Song_J2ME[] out = new Song_J2ME[length];
		for (int i = 0; i < length; i++) {
			out[i] = db_lnk.songs[i + offset];
		}
		return out;
	}

	/**
	 * 
	 * delete song in server playlist
	 * 
	 * @param pos
	 *            position
	 * 
	 * */
	public void delete(int pos) {
		sendCommand(CMD_Enum.CMD_DELETE, "", Integer.toString(pos));
	}

	/**
	 * 
	 * play next song
	 * 
	 * */
	public void next() {
		sendCommand(CMD_Enum.CMD_NEXT, "", "");
	}

	/**
	 * 
	 * play by position in playlist
	 * 
	 * @param index
	 *            position
	 * 
	 * */
	public void play(int index) {
		sendCommand(CMD_Enum.CMD_PLAY_BY_POS, "", Integer.toString(index));
	}

	/**
	 * 
	 * pause or resume
	 * 
	 * */
	public void pause_resume() {
		sendCommand(CMD_Enum.CMD_PAUSE, "", "");
	}

	/**
	 * 
	 * play by id
	 * 
	 * @param id
	 * 
	 * */
	public void playId(int id) {
		sendCommand(CMD_Enum.CMD_PLAY_BY_ID, "", "");
	}

	/**
	 * 
	 * go back
	 * 
	 * */

	public void prev() {
		sendCommand(CMD_Enum.CMD_PREVIOS, "", "");
	}

	/**
	 * set volume
	 * 
	 * 0-100
	 * 
	 * @param vol
	 */

	public void setVolume(int vol) {
		sendCommand(CMD_Enum.CMD_SET_VOLUME, "", Integer.toString(vol));
	}

	/**
	 * 
	 * stop playing
	 * 
	 * */
	public void stop() {
		sendCommand(CMD_Enum.CMD_STOP, "", "");
	}

	/**
	 * 
	 * update database(playlist) from server
	 * 
	 * */
	public void updateDb() {
		sendCommand(CMD_Enum.CMD_GET_PLAYLIST, "", "");
	}

	public void save(String key, String[] val) {
		storage_lnk.save(key, MasterExtender_J2ME.StringArrayToByteArray(val));
	}

	private Song_J2ME[] parseStringsToSongs(String[] all) {
		System.out.println("parsing " + all.length);
		Song_J2ME[] out = new Song_J2ME[all.length / QueryFilter_J2ME.TAG_SIZE];
		for (int i = 0; i < all.length / QueryFilter_J2ME.TAG_SIZE; i++) {
			out[i] = new Song_J2ME();
			System.out.println("songc:" + i);
			out[i].title = all[QueryFilter_J2ME.TITLE_POSITION + i
					* QueryFilter_J2ME.TAG_SIZE].substring(TITLE.length());
			out[i].file = all[QueryFilter_J2ME.FILE_POSITION + i
					* QueryFilter_J2ME.TAG_SIZE].substring(FILE.length());
			out[i].duration = Integer.valueOf(
					0 + all[QueryFilter_J2ME.TIME_POSITION + i
							* QueryFilter_J2ME.TAG_SIZE].substring(TIME
							.length())).intValue();
			out[i].artist = all[QueryFilter_J2ME.ARTIST_POSITION + i
					* QueryFilter_J2ME.TAG_SIZE].substring(ARTIST.length());
			out[i].album = all[QueryFilter_J2ME.ALBUM_POSITION + i
					* QueryFilter_J2ME.TAG_SIZE].substring(ALBUM.length());
			out[i].date = all[QueryFilter_J2ME.YEAR_POSITION + i
					* QueryFilter_J2ME.TAG_SIZE].substring(YEAR.length());
			if (out[i].title.compareTo("") == 0)
				out[i].title = out[i].file;
		}
		return out;
	}
}
